import ErrorBoundary from './ErrorBoundary';
import Identity from './StatsigIdentity';
import StatsigSDKOptions from './StatsigSDKOptions';
import { StatsigUser } from './StatsigUser';
export declare enum StatsigEndpoint {
    Initialize = "initialize",
    Rgstr = "rgstr",
    LogEventBeacon = "log_event_beacon"
}
type NetworkResponse = Response & {
    data?: Record<string, unknown>;
};
/**
 * An extension of the promise type, it adds a
 * function `eventually`. In the event that the provided timeout
 * is reached, the function will still be called regardless.
 *
 * This function WILL NOT BE CALLED if the promise resolves normally.
 */
type PromiseWithTimeout<T> = Promise<T> & {
    eventually: (fn: (t: T) => void) => PromiseWithTimeout<T>;
};
export default class StatsigNetwork {
    private readonly _options;
    private readonly _identity;
    private readonly _errorBoundary;
    private readonly retryCodes;
    private leakyBucket;
    private canUseKeepalive;
    constructor(options: StatsigSDKOptions, identity: Identity, errorBoundary: ErrorBoundary);
    fetchValues(user: StatsigUser | null, sinceTime: number | null, timeout: number, previousDerivedFields?: Record<string, string>): PromiseWithTimeout<Record<string, any>>;
    sendLogBeacon(payload: Record<string, any>): boolean;
    postToEndpoint(endpointName: StatsigEndpoint, body: object, retries?: number, backoff?: number, retryAttempt?: number, useKeepalive?: boolean): Promise<NetworkResponse>;
    supportsKeepalive(): boolean;
    private _init;
    private _postWithTimeout;
    private _getErrorData;
}
export {};
