import DynamicConfig from './DynamicConfig';
import Layer from './Layer';
import { LocalOverrides } from './LocalOverrides';
import { StatsigOptions } from './StatsigSDKOptions';
import { EvaluationDetails } from './StatsigStore';
import { StatsigUser } from './StatsigUser';
export { default as DynamicConfig } from './DynamicConfig';
export { default as Layer } from './Layer';
export { StatsigEnvironment, StatsigOptions } from './StatsigSDKOptions';
export { EvaluationReason } from './StatsigStore';
export type { EvaluationDetails } from './StatsigStore';
export { StatsigUser } from './StatsigUser';
export type { LocalOverrides };
export default class Statsig {
    private static instance;
    static initialize(sdkKey: string, user?: StatsigUser | null, options?: StatsigOptions | null): Promise<void>;
    static setInitializeValues(initializeValues: Record<string, unknown>): void;
    static checkGate(gateName: string): boolean;
    static checkGateWithExposureLoggingDisabled(gateName: string): boolean;
    static manuallyLogGateExposure(gateName: string): void;
    static getConfig(configName: string): DynamicConfig;
    static getConfigWithExposureLoggingDisabled(configName: string): DynamicConfig;
    static manuallyLogConfigExposure(configName: string): void;
    static getExperiment(experimentName: string): DynamicConfig;
    static getExperimentWithExposureLoggingDisabled(experimentName: string): DynamicConfig;
    static manuallyLogExperimentExposure(configName: string): void;
    static getLayer(layerName: string): Layer;
    static getLayerWithExposureLoggingDisabled(layerName: string): Layer;
    static manuallyLogLayerParameterExposure(layerName: string, parameterName: string): void;
    static logEvent(eventName: string, value?: string | number | null, metadata?: Record<string, string> | null): void;
    static updateUser(user: StatsigUser | null): Promise<boolean>;
    static updateUserWithValues(user: StatsigUser | null, values: Record<string, unknown>): boolean;
    static shutdown(): void;
    static overrideGate(gate: string, result: boolean | null): void;
    static overrideConfig(config: string, result: Record<string, unknown> | null): void;
    static overrideLayer(layer: string, result: Record<string, unknown> | null): void;
    static setOverrides(overrides: LocalOverrides | null): void;
    static getOverrides(): LocalOverrides;
    /**
     * @returns The Statsig stable ID used for device level experiments
     */
    static getStableID(): string;
    /**
     *
     * @returns The reason and time associated with the evaluation for the current set
     * of gates and configs
     */
    static getEvaluationDetails(): EvaluationDetails;
    /**
     *
     * @returns true if initialize has already been called, false otherwise
     */
    static initializeCalled(): boolean;
    private static _getClientX;
}
